package de.ids_mannheim.korap.client;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.view.RedirectView;

@Service
public class OAuth2Service {

    @Value("${client.id}")
    private String clientId;
    @Value("${client.secret}")
    private String clientSecret;
    
    @Value("${oauth.scope}")
    private String scope;

    @Value("${korap.authorize.url}")
    private String korapAuthorizeUrl;
    @Value("${korap.token.url}")
    private String korapTokenUrl;

    @Value("${client.redirect.uri}")
    private String redirectUri;
    
    public RedirectView createRedirect () throws UnsupportedEncodingException {
        String encodedURL = URLEncoder.encode(redirectUri, "UTF-8");
        String locationUri = korapAuthorizeUrl + "/authorize?client_id="
                + clientId + "&redirect_uri=" + encodedURL
                + "&scope=" + scope
                + "&response_type=code&state=ZMwDGTZ2RY";

        RedirectView redirectView = new RedirectView();
        redirectView.setUrl(locationUri);
        return redirectView;
    }

    public String requestToken (String code, String state) {
        List<MediaType> list = new ArrayList<MediaType>();
        list.add(MediaType.APPLICATION_JSON);
        
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        headers.setAccept(list);

        String tokenRequestParams = "client_id=" + clientId + "&client_secret="
                + clientSecret + "&grant_type=authorization_code&redirect_uri="
                + redirectUri + "&code=" + code;
        
        HttpEntity<String> request = new HttpEntity<String>(tokenRequestParams, headers);
        
        RestTemplate restTemplate = new RestTemplate();
        String result = restTemplate.postForObject(korapTokenUrl, request, String.class);
        return result;
    }
}
